<?php

namespace App\HttpController\Street;

use App\Key\K;
use App\Model\Street\ConfigList;
use App\Model\Street\DataTableList;
use App\Model\Street\DownloadList;
use App\Model\Street\ErrorList;
use App\Tools\System;
use EasySwoole\Component\TableManager;
use EasySwoole\EasySwoole\Config;
use EasySwoole\Mysqli\Client;
use EasySwoole\ORM\DbManager;

class Work extends Base
{
    public function index()
    {
        $outDate = [];
        $confirmTable = TableManager::getInstance()->get(K::TABLE_STREET_ID_CONFIRM_CACHE);
        //注意，这个ID，不能给的太大了，要不然子服务器无法处理大量的数据
        $id_limit = 20;
        foreach ($confirmTable as $k => $v) {
            if (time() - $v['time'] > 600) {
                //注意，里面的数据也是一个数组，要和下面的对应起来
                $outDate[] = ['id' => $k];
                if (count($outDate) > $id_limit) break;
            }
        }

        foreach ($outDate as $v) {
            //将这一部分的数据重新放到缓存中，进行处理
            $confirmTable->set($v['id'], ['time' => time()]);
        }

        if ($outDate) {
            //如果有缓存的数据 ，先使用缓存的数据
            return $this->jsonSuccess(['ids' => $outDate]);
        }

        System::log('cache table count: ' . $confirmTable->count());
        if ($confirmTable->count() > 800) {
            //如果缓存中已经存在太多的数据，不会返回
            System::log('too many cache ids wait...');
            return $this->jsonSuccess(['ids' => []]);
        }
        $nextTable = DataTableList::getNextTableName();
        if (!$nextTable) {
            System::log('no work to do');
            return $this->jsonSuccess(['ids' => []]);
        }

        $dbConfig = Config::getInstance()->getConf('street');
        $config = new \EasySwoole\Mysqli\Config($dbConfig);
        $client = new Client($config);
        $configModel = new ConfigList();
        $next_id = $configModel->getNextId();
        if (!$next_id) {
            $client->queryBuilder()->orderBy('id', 'ASC')->limit($id_limit)->fields(['id'])->get($nextTable['name']);
            System::log('table name ' . $nextTable['name'] . ' no next_id');
        } else {
            System::log('table name ' . $nextTable['name'] . ' next_id ' . $next_id);
            $client->queryBuilder()->where('id', $next_id, '>')->orderBy('id', 'ASC')->limit($id_limit)->fields(['id'])->get($nextTable['name']);
        }
        $ids = $client->execBuilder();
        foreach ($ids as $id) {
            $next_id = $id['id'];
            $confirmTable->set($id['id'], ['time' => time()]);
        }
        System::log('current worker next_id :'.$next_id);
        $dataModel = new DataTableList();
        if (count($ids) < $id_limit) {
            //如果已经少于20个ID，那么，需要重新生成
            System::log('update next_id to empty new table');
            $configModel->updateNextId('');
            $dataModel->where(['name' => $nextTable['name']])->update(['done' => 1, 'done_time' => time()]);
        } else {
            $configModel->updateNextId($next_id);
        }
        return $this->jsonSuccess(['ids' => $ids]);
    }

    public function confirm()
    {
        //客户端确认数据已经下载完成，那么将缓存中的数据要删除掉
        $data = $this->getRequestData();
        $table = TableManager::getInstance()->get(K::TABLE_STREET_ID_CONFIRM_CACHE);
        //这里返回的结果也是有下标的，要对应起来
        foreach ($data['ids'] as $v) {
            $table->del($v['id']);
        }
        System::log('confirm ids size : ', count($data['ids']));
        return $this->jsonSuccess([]);
    }
    //子服务器将下载好的数据提交到这里来
    public function db()
    {
        $data = $this->getRequestData();
        $model = new DownloadList();
        DbManager::getInstance()->startTransaction('street');
        $del = [];
        foreach ($data['dbs'] as $v) {
            $has = $model->where(['id' => $v])->get();
            if ($has) {
                if ($has['is_done']) {
                    $model->where(['id' => $v])->update(['is_delete' => 1]);
                    $del[] = $v;
                }
            } else {
                $obj = $model::create(['id' => $v, 'ip' => $this->ip]);
                $obj->save();
            }
        }
        DbManager::getInstance()->commit('street');
        return $this->jsonSuccess(['del' => $del]);
    }

    public function download()
    {
        $model = new DownloadList();
        switch ($this->request()->getMethod()) {
            case 'POST':
            case 'GET':
                $data = $this->getRequestData();
                if (!$data['id']) {
                    $result = $model->order('id', 'ASC')->limit(20)->all();
                } else {
                    $result = $model->where('id', $data['id'], '>')->order('id', 'ASC')->limit(20)->all();
                }
                $result = array_map(function ($v) {
                    return $v->toArray();
                }, $result);
                return $this->jsonSuccess($result);
                break;
            case 'PUT':
                $data = $this->getRequestData();
                if (!$data['id']) return $this->jsonError('no id');
                $result=$model->groupSave($data['id'],'edit_download',['is_done' => 1, 'download_time' => time()]);
                if (!$result) return $this->jsonError($model->getError());
                return $this->jsonSuccess($result);
                break;
        }
    }

    public function error()
    {
        $data = $this->getRequestData();
        $model = new ErrorList();
        DbManager::getInstance()->startTransaction('street');
        foreach ($data as $v) {
            $has = $model->where(['id' => $v['id']])->get();
            if (!$has) {
                $data_str = is_array($v['data']) ? json_encode($v['data']) : $v['data'];
                $obj = $model::create(['id' => $v['id'], 'data' => $data_str]);
                $obj->save();
            }
        }
        DbManager::getInstance()->commit('street');
        return $this->jsonSuccess(['ok']);
    }
}
