<?php


namespace EasySwoole\EasySwoole;


use App\HttpController\Google\Handler;
use App\HttpController\Google\WebSocket;
use App\Key\K;
use App\Tools\System;
use EasySwoole\Component\AtomicManager;
use EasySwoole\Component\Di;
use EasySwoole\Component\TableManager;
use EasySwoole\EasySwoole\AbstractInterface\Event;
use EasySwoole\EasySwoole\Swoole\EventRegister;
use EasySwoole\HotReload\HotReload;
use EasySwoole\HotReload\HotReloadOptions;
use EasySwoole\Http\Request;
use EasySwoole\Http\Response;
use EasySwoole\ORM\Db\Connection;
use EasySwoole\ORM\DbManager;
use EasySwoole\Pool\Exception\Exception;
use EasySwoole\Trigger\Location;
use Swoole\Table;

class EasySwooleEvent implements Event
{
    public static function initialize()
    {
        date_default_timezone_set('Asia/Shanghai');
        Di::getInstance()->set(SysConst::ERROR_REPORT_LEVEL, E_ERROR);
        Di::getInstance()->set(SysConst::ERROR_HANDLER, function ($errorCode, $description, $file = null, $line = null) {
            if (!in_array($errorCode, [E_ERROR], true)) return;
//            if (!in_array($errorCode,[E_ERROR,E_WARNING],true)) return;
            $l = new Location();
            $l->setFile($file);
            $l->setLine($line);
            Trigger::getInstance()->error($description, $errorCode, $l);
        });
        Di::getInstance()->set(SysConst::HTTP_GLOBAL_ON_REQUEST, function (Request $request, Response $response) {
            $response->getSwooleResponse()->header('server', 'bigemap-server');
            $response->withAddedHeader('server', 'bigemap-server');
            $serverId = System::getServerIdNumber();
            $response->getSwooleResponse()->header('server-uu', $serverId);
            $response->withAddedHeader('server-uu', $serverId);
            return true;
        });
    }

    public static function mainServerCreate(EventRegister $register)
    {
        static::initTable();
        static::initTableData();
        if (file_exists(dirname(EASYSWOOLE_ROOT).'/manager/main_server')){
            echo 'main server',PHP_EOL;
            static::createConnect();
            $config=Config::getInstance()->getConf('MAIN_SERVER');
            if ($config['PORT']=='9402'){
                $hotReloadOptions = new HotReloadOptions();
                $hotReloadOptions->setMonitorFolder([EASYSWOOLE_ROOT . '/App']);
                $hotReload = new HotReload($hotReloadOptions);
                $server = ServerManager::getInstance()->getSwooleServer();
                $hotReload->attachToServer($server);
            }
        }
        $handler = new Handler();
        $register->set(EventRegister::onWorkerStart, [$handler, 'workerStart']);
        $register->set(EventRegister::onPipeMessage,[$handler,'pipeMessage']);
        // 自定义握手
        $socket = new \App\WebSocket\WebSocket();
        $register->set(EventRegister::onHandShake, [$socket, 'onHandShake']);
        $register->set(EventRegister::onMessage, [$socket, 'message']);
        $register->set(EventRegister::onOpen, [$socket, 'open']);
        $register->set(EventRegister::onClose, [$socket, 'close']);
    }
    public static function createConnect(){
        $config = new \EasySwoole\ORM\Db\Config(Config::getInstance()->getConf('street'));
        try {
            //配置连接池最大数量
            $config->setMaxObjectNum(20);
            $config->setMinObjectNum(1);
//            $config->setReturnCollection(true);
            DbManager::getInstance()->addConnection(new Connection($config),'street');
        } catch (Exception $e) {
            echo $e->getMessage(),PHP_EOL;
        }

        $config = new \EasySwoole\ORM\Db\Config(Config::getInstance()->getConf('history'));
        try {
            //配置连接池最大数量
            $config->setMaxObjectNum(20);
            $config->setMinObjectNum(1);
//            $config->setReturnCollection(true);
            DbManager::getInstance()->addConnection(new Connection($config),'history');
        } catch (Exception $e) {
            echo $e->getMessage(),PHP_EOL;
        }
    }

    public static function initTable()
    {
        TableManager::getInstance()->add(K::TABLE_FIRST_CACHE, ['data' => ['type' => Table::TYPE_STRING, 'size' => 1024], 'sum' => ['type' => Table::TYPE_INT], 'time' => ['type' => Table::TYPE_INT, 'size' => 4]]);
        TableManager::getInstance()->add(K::TABLE_SECOND_CACHE, ['data' => ['type' => Table::TYPE_STRING, 'size' => 1024], 'sum' => ['type' => Table::TYPE_INT], 'time' => ['type' => Table::TYPE_INT, 'size' => 4]]);
        $column = [
            'first_time' => ['type' => Table::TYPE_INT],
            'first_data' => ['type' => Table::TYPE_STRING, 'size' => 1024],
            'first_header' => ['type' => Table::TYPE_STRING, 'size' => 1024],
            'second_time' => ['type' => Table::TYPE_INT],
            'second_data' => ['type' => Table::TYPE_STRING, 'size' => 1024],
            'second_header' => ['type' => Table::TYPE_STRING, 'size' => 1024],
        ];
        TableManager::getInstance()->add(K::TABLE_QUERY_LIST, $column);
        TableManager::getInstance()->add(K::TABLE_IP_400, ['number' => ['type' => Table::TYPE_INT], 'time' => ['type' => Table::TYPE_INT]], 1024);
        TableManager::getInstance()->add(K::TABLE_WEB_FD, ['data' => ['type' => Table::TYPE_STRING, 'size' => 512]]);
        TableManager::getInstance()->add(K::TABLE_WHITE_LIST, ['data' => ['type' => Table::TYPE_INT]]);
//        TableManager::getInstance()->add(K::TABLE_BLACK_LIST, ['data' => ['type' => Table::TYPE_INT]], 1024 * 4);
        TableManager::getInstance()->add(K::TABLE_QUERY_WAITING, ['time'=>['type'=>Table::TYPE_FLOAT],'worker_id'=>['type'=>Table::TYPE_INT],'cid'=>['type'=>Table::TYPE_FLOAT],'code' => ['type' => Table::TYPE_INT]]);
        $config=Config::getInstance()->getConf('BIGEMAP_DATA');
        if ($config['street_view']){
            TableManager::getInstance()->add(K::TABLE_STREET_CURL_URL_CACHE, ['data'=>['type'=>Table::TYPE_STRING,'size'=>256]],1024*400);
            TableManager::getInstance()->add(K::TABLE_STREET_DATA_CACHE, ['data'=>['type'=>Table::TYPE_STRING,'size'=>1024*200]],K::TABLE_SIZE_STREET_DATA);
        }
        //这里主要是要判断如果是内网中的主服务器，那么这个长度要增加，因为内网中有很多的子服务器；
        $cache=dirname(__DIR__).'/cache/9502/config';
        $cache=json_decode(file_get_contents($cache),true);
        if (in_array($cache[K::KEY_SERVER_ID],K::CONST_INNER_SERVER_ID)){
            $size=1024*10;
        }else{
            $size=1024*2;
        }
        file_put_contents(__DIR__.'/table_size',$size);
        TableManager::getInstance()->add(K::TABLE_SYSTEM_CONFIG, ['data' => ['type' => Table::TYPE_STRING, 'size' => $size], 'number' => ['type' => Table::TYPE_FLOAT]],128);
        TableManager::getInstance()->add(K::TABLE_STREET_ID_CONFIRM_CACHE, ['time' => ['type' => Table::TYPE_INT]]);
    }

    public static function initTableData()
    {
        //如果重启了还原每天的数据量
        $path = System::config(K::KEY_CACHE_PATH);
        foreach ([K::KEY_EVERY_DAY_UPLOAD, K::KEY_EVERY_DAY_POST1, K::KEY_EVERY_DAY_POST2] as $v) {
            $data = file_get_contents($path . '/' . $v);
            $data = json_decode($data, true);
            if ($data) {
                TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG)->set($v, $data);
            }
        }
    }
}
